package sample;

import javafx.beans.property.ReadOnlyStringWrapper;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.*;
import javafx.stage.FileChooser;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.ResourceBundle;

public class Controller implements Initializable{
    @FXML private Button buttonSelectPath;
    @FXML private TextField textFieldPath;
    @FXML private TextField textFieldNumbers;
    @FXML private TextField textFieldBefore;
    @FXML private TextField textFieldBehind;
    @FXML private Button buttonSearch;
    @FXML private TableView<ArrayList<ArrayList<Integer>>> tableView;
    @FXML private Label labelStatus;
    @Override
    public void initialize(URL location, ResourceBundle resources) {
        //textFieldNumbers.setText("27;20;5;10");
        //textFieldPath.setText("C:\\Users\\Patrick\\Desktop\\Archiv-Test.csv");
        buttonSelectPath.setOnAction(event -> {
            File tmp=selectFile();
            if(tmp!=null)
                textFieldPath.setText(tmp.getAbsolutePath());
        });
        buttonSearch.setOnAction(event -> {
            buttonSearch.setDisable(true);
            File file=getFile();
            if(file==null) {
                buttonSearch.setDisable(false);
                return;
            }
            int before=getBefore();
            if(before<0) {
                buttonSearch.setDisable(false);
                return;
            }
            int behind=getBehind();
            if(behind<0) {
                buttonSearch.setDisable(false);
                return;
            }
            int[] searchNumbers=getSearchNumbers();
            if(searchNumbers==null) {
                buttonSearch.setDisable(false);
                return;
            }
            Table table=new Table(file);
            ArrayList<ArrayList<ArrayList<Integer>>> result=table.searchTable(this,before,behind,searchNumbers);
            fillTableView(result);
            buttonSearch.setDisable(false);
        });
    }
    public void setStatus(String status){
        labelStatus.setText(status);
    }
    public File selectFile(){
        FileChooser fileChooser=new FileChooser();
        fileChooser.setTitle("Whle CSV-Datei!");
        FileChooser.ExtensionFilter tmp=new FileChooser.ExtensionFilter("CSV-Datei (*.csv)","*.csv");
        fileChooser.getExtensionFilters().add(tmp);
        fileChooser.setSelectedExtensionFilter(tmp);
        return fileChooser.showOpenDialog(Main.primaryStage);
    }
    public File getFile(){
        File tmp=new File(textFieldPath.getText());
        if(!tmp.exists()) {
            showError("Datei existiert nicht","Die angegebene Datei: \""+textFieldPath.getText()+"\" existiert nicht!");
            return null;
        }
        return tmp;
    }
    public int getBefore(){
        int number;
        try {
            number=Integer.parseInt(textFieldBefore.getText());
        }catch(NumberFormatException e){
            showError("Keine Zahl","In das Feld fr die Anzahl Zahlen vor dem Ergebnis wurde keine Zahl eingetragen.");
            return -1;
        }
        if(number<0)
            showError("Falsche Zahl","Die Anzahl Zahlen vor der gesuchten muss 0 oder Positiv sein!");
        return number;
    }
    public int getBehind(){
        int number;
        try {
            number=Integer.parseInt(textFieldBehind.getText());
        }catch(NumberFormatException e){
            showError("Keine Zahl","In das Feld fr die Anzahl Zahlen nach dem Ergebnis wurde keine Zahl eingetragen.");
            return -1;
        }
        if(number<0)
            showError("Falsche Zahl","Die Anzahl Zahlen nach der gesuchten muss 0 oder Positiv sein!");
        return number;
    }
    public int[] getSearchNumbers(){
        try {
            String[] sNumbers = textFieldNumbers.getText().split(";");
            int[] tmp = new int[sNumbers.length];
            for (int i = 0; i < sNumbers.length; i++)
                tmp[i] = Integer.parseInt(sNumbers[i]);
            return tmp;
        }catch (java.lang.NumberFormatException e){
            showError("Die Eingabe ist nicht lesbar!","Es drfen nur Zahlen mit ; getrennt in das Suchzahlen-Feld eingegeben werden!");
            return null;
        }
    }
    public void fillTableView(ArrayList<ArrayList<ArrayList<Integer>>> lists){
        tableView.getItems().clear();
        tableView.getColumns().clear();

        for (int i = tableView.getColumns().size(); i < lists.size(); i++) {
            TableColumn<ArrayList<ArrayList<Integer>>, String> col = new TableColumn<>("Column "+(i+1));
            col.setMinWidth(80);
            final int colIndex = i ;
            col.setCellValueFactory(data -> {
                ArrayList<ArrayList<Integer>> rowValues = data.getValue();
                String cellValue ;
                if (colIndex < rowValues.size()) {
                    if(rowValues.get(colIndex)==null)
                        cellValue="";
                    else
                        cellValue = ""+rowValues.get(colIndex);
                } else {
                    cellValue = "" ;
                }
                return new ReadOnlyStringWrapper(cellValue);
            });
            tableView.getColumns().add(col);
        }
        int maxValues=0;
        for(int i=0;i<lists.size();i++)
            if(lists.get(i).size()>maxValues)
                maxValues=lists.get(i).size();
        for(int i=0;i<maxValues;i++){
            ArrayList<ArrayList<Integer>> tmp=new ArrayList<>();
            for(int j=0;j<lists.size();j++){
                if(i<lists.get(j).size())
                    tmp.add(lists.get(j).get(i));
            }
            tableView.getItems().add(tmp);
        }
    }
    public void showError(String title,String message){
        Alert alert = new Alert(Alert.AlertType.ERROR);
        alert.setTitle("Error Dialog");
        alert.setHeaderText(title);
        alert.setContentText(message);
        alert.showAndWait();
        System.out.println(title+": "+message);
    }
}
